using System;
using System.IO;
using System.Text;
using System.Reflection;
using System.Collections;
using Server;

namespace Server.Scripts.Commands
{
	public class ObjectList
	{
		public static void Initialize()
		{
			Server.Commands.Register( "ObjList", AccessLevel.Administrator, new CommandEventHandler( ObjList_OnCommand ) );
		}

		private static void ObjList_OnCommand( CommandEventArgs e )
		{
			World.Broadcast( 0x35, true, "Object lists are being generated, please wait." );

			DateTime startTime = DateTime.Now;

			Generate();

			DateTime endTime = DateTime.Now;

			World.Broadcast( 0x35, true, "Generation has been completed. The entire process took {0:F1} seconds.", (endTime - startTime).TotalSeconds );
		}

		private static string[,] m_Aliases = new string[,]
			{
				{ "System.Object",	"<font color=\"blue\">object</font>" },
				{ "System.String",	"<font color=\"blue\">string</font>" },
				{ "System.Boolean",	"<font color=\"blue\">bool</font>" },
				{ "System.Byte",	"<font color=\"blue\">byte</font>" },
				{ "System.SByte",	"<font color=\"blue\">sbyte</font>" },
				{ "System.Int16",	"<font color=\"blue\">short</font>" },
				{ "System.UInt16",	"<font color=\"blue\">ushort</font>" },
				{ "System.Int32",	"<font color=\"blue\">int</font>" },
				{ "System.UInt32",	"<font color=\"blue\">uint</font>" },
				{ "System.Int64",	"<font color=\"blue\">long</font>" },
				{ "System.UInt64",	"<font color=\"blue\">ulong</font>" },
				{ "System.Single",	"<font color=\"blue\">float</font>" },
				{ "System.Double",	"<font color=\"blue\">double</font>" },
				{ "System.Decimal",	"<font color=\"blue\">decimal</font>" },
				{ "System.Char",	"<font color=\"blue\">char</font>" },
				{ "System.Void",	"<font color=\"blue\">void</font>" }
			};

		private static int m_AliasLength = m_Aliases.GetLength( 0 );

		public static string GetPair( Type varType, string name )
		{
			StringBuilder append = new StringBuilder();

			Type realType = varType;

			if ( realType.IsPointer )
			{
				if ( realType.IsArray )
				{
					append.Append( '*' );

					do
					{
						append.Append( '[' );

						for ( int i = 1; i < realType.GetArrayRank(); ++i )
							append.Append( ',' );

						append.Append( ']' );

						realType = realType.GetElementType();
					} while ( realType.IsArray );

					append.Append( ' ' );
				}
				else
				{
					realType = realType.GetElementType();
					append.Append( " *" );
				}
			}
			else if ( realType.IsArray )
			{
				do
				{
					append.Append( '[' );

					for ( int i = 1; i < realType.GetArrayRank(); ++i )
						append.Append( ',' );

					append.Append( ']' );

					realType = realType.GetElementType();
				} while ( realType.IsArray );

				append.Append( ' ' );
			}
			else
			{
				append.Append( ' ' );
			}

			string fullName = realType.FullName;
			string aliased = null;

			for ( int i = 0; i < m_AliasLength; ++i )
			{
				if ( m_Aliases[i, 0] == fullName )
				{
					aliased = m_Aliases[i, 1];
					break;
				}
			}

			if ( aliased == null )
				aliased = realType.Name;

			return String.Concat( aliased, append, name );
		}

		private class TypeComparer : IComparer
		{
			public int Compare( object x, object y )
			{
				Type a = x as Type;
				Type b = y as Type;

				return a.Name.CompareTo( b.Name );
			}
		}

		private static bool IsConstructable( ConstructorInfo ctor )
		{
			return ctor.IsDefined( typeof( ConstructableAttribute ), false );
		}

		public static void Generate()
		{
			using ( StreamWriter op = new StreamWriter( "objlist.html" ) )
			{
				op.WriteLine( "<html>" );
				op.WriteLine( "   <head>" );
				op.WriteLine( "      <title>RunUO Object List</title>" );
				op.WriteLine( "   </head>" );
				op.WriteLine( "   <body bgcolor=\"white\" style=\"font-family: Courier New\" text=\"#000000\" link=\"#000000\" vlink=\"#000000\" alink=\"#808080\">" );
				op.WriteLine( "      <h2>Objects</h2>" );

				Type[] allTypes = ScriptCompiler.Assembly.GetTypes();

				Array.Sort( allTypes, new TypeComparer() );

				for ( int i = 0; i < allTypes.Length; ++i )
				{
					Type t = allTypes[i];

					ConstructorInfo[] ctors = t.GetConstructors();

					for ( int j = 0; j < ctors.Length; ++j )
					{
						ConstructorInfo ctor = ctors[j];

						if ( IsConstructable( ctor ) )
						{
							op.Write( "      " );
							op.Write( t.Name );
							op.Write( '(' );

							ParameterInfo[] parms = ctor.GetParameters();

							if ( parms.Length > 0 )
							{
								op.Write( ' ' );

								for ( int k = 0; k < parms.Length; ++k )
								{
									ParameterInfo pi = parms[k];

									if ( k != 0 )
										op.Write( ", " );

									op.Write( GetPair( pi.ParameterType, pi.Name ) );
								}

								op.Write( ' ' );
							}

							op.WriteLine( ")<br>" );
						}
					}
				}

				op.WriteLine( "   </body>" );
				op.WriteLine( "</html>" );
			}
		}
	}
}